@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -32,6 +32,7 @@ import dateutil.parser
 import dateutil.tz
 
 from barman import output, xlog
+from barman.annotations import KeepManager, KeepManagerMixin
 from barman.backup_executor import (
     PassiveBackupExecutor,
     PostgresBackupExecutor,
@@ -44,11 +45,13 @@ from barman.exceptions import (
     CompressionIncompatibility,
     SshCommandException,
     UnknownBackupIdException,
+    CommandFailedException,
 )
+from barman.fs import unix_command_factory
 from barman.hooks import HookScriptRunner, RetryHookScriptRunner
 from barman.infofile import BackupInfo, LocalBackupInfo, WalFileInfo
 from barman.lockfile import ServerBackupSyncLock
-from barman.recovery_executor import RecoveryExecutor
+from barman.recovery_executor import recovery_executor_factory
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import (
     force_str,
@@ -57,11 +60,12 @@ from barman.utils import (
     human_readable_timedelta,
     pretty_size,
 )
+from barman.command_wrappers import PgVerifyBackup
 
 _logger = logging.getLogger(__name__)
 
 
-class BackupManager(RemoteStatusMixin):
+class BackupManager(RemoteStatusMixin, KeepManagerMixin):
     """Manager of the backup archive for a server"""
 
     DEFAULT_STATUS_FILTER = BackupInfo.STATUS_COPY_DONE
@@ -69,8 +73,9 @@ class BackupManager(RemoteStatusMixin):
     def __init__(self, server):
         """
         Constructor
+        :param server: barman.server.Server
         """
-        super(BackupManager, self).__init__()
+        super(BackupManager, self).__init__(server=server)
         self.server = server
         self.config = server.config
         self._backup_cache = None
@@ -178,6 +183,25 @@ class BackupManager(RemoteStatusMixin):
             return available_backups.get(backup_id)
         return None
 
+    @staticmethod
+    def find_previous_backup_in(
+        available_backups, backup_id, status_filter=DEFAULT_STATUS_FILTER
+    ):
+        """
+        Find the next backup (if any) in the supplied dict of BackupInfo objects.
+        """
+        ids = sorted(available_backups.keys())
+        try:
+            current = ids.index(backup_id)
+            while current > 0:
+                res = available_backups[ids[current - 1]]
+                if res.status in status_filter:
+                    return res
+                current -= 1
+            return None
+        except ValueError:
+            raise UnknownBackupIdException("Could not find backup_id %s" % backup_id)
+
     def get_previous_backup(self, backup_id, status_filter=DEFAULT_STATUS_FILTER):
         """
         Get the previous backup (if any) in the catalog
@@ -189,14 +213,123 @@ class BackupManager(RemoteStatusMixin):
             status_filter = tuple(status_filter)
         backup = LocalBackupInfo(self.server, backup_id=backup_id)
         available_backups = self.get_available_backups(status_filter + (backup.status,))
+        return self.find_previous_backup_in(available_backups, backup_id, status_filter)
+
+    @staticmethod
+    def should_remove_wals(
+        backup,
+        available_backups,
+        keep_manager,
+        skip_wal_cleanup_if_standalone,
+        status_filter=DEFAULT_STATUS_FILTER,
+    ):
+        """
+        Determine whether we should remove the WALs for the specified backup.
+
+        Returns the following tuple:
+
+           - `(bool should_remove_wals, list wal_ranges_to_protect)`
+
+        Where `should_remove_wals` is a boolean which is True if the WALs associated
+        with this backup should be removed and False otherwise.
+
+        `wal_ranges_to_protect` is a list of `(begin_wal, end_wal)` tuples which define
+        *inclusive* ranges where any matching WAL should not be deleted.
+
+        The rules for determining whether we should remove WALs are as follows:
+
+          1. If there is no previous backup then we can clean up the WALs.
+          2. If there is a previous backup and it has no keep annotation then do
+             not clean up the WALs. We need to allow PITR from that older backup
+             to the current time.
+          3. If there is a previous backup and it has a keep target of "full" then
+             do nothing. We need to allow PITR from that keep:full backup to the
+             current time.
+          4. If there is a previous backup and it has a keep target of "standalone":
+            a. If that previous backup is the oldest backup then delete WALs up to
+               the begin_wal of the next backup except for WALs which are
+               >= begin_wal and <= end_wal of the keep:standalone backup - we can
+               therefore add `(begin_wal, end_wal)` to `wal_ranges_to_protect` and
+               return True.
+            b. If that previous backup is not the oldest backup then we add the
+               `(begin_wal, end_wal)` to `wal_ranges_to_protect` and go to 2 above.
+               We will either end up returning False, because we hit a backup with
+               keep:full or no keep annotation, or all backups to the oldest backup
+               will be keep:standalone in which case we will delete up to the
+               begin_wal of the next backup, preserving the WALs needed by each
+               keep:standalone backups by adding them to `wal_ranges_to_protect`.
+
+        This is a static method so it can be re-used by barman-cloud which will
+        pass in its own dict of available_backups.
+
+        :param BackupInfo backup_info: The backup for which we are determining
+          whether we can clean up WALs.
+        :param dict[str,BackupInfo] available_backups: A dict of BackupInfo
+          objects keyed by backup_id which represent all available backups for
+          the current server.
+        :param KeepManagerMixin keep_manager: An object implementing the
+          KeepManagerMixin interface. This will be either a BackupManager (in
+          barman) or a CloudBackupCatalog (in barman-cloud).
+        :param bool skip_wal_cleanup_if_standalone: If set to True then we should
+          skip removing WALs for cases where all previous backups are standalone
+          archival backups (i.e. they have a keep annotation of "standalone").
+          The default is True. It is only safe to set this to False if the backup
+          is being deleted due to a retention policy rather than a `barman delete`
+          command.
+        :param status_filter: The status of the backups to check when determining
+          if we should remove WALs. default to DEFAULT_STATUS_FILTER.
+        """
+        previous_backup = BackupManager.find_previous_backup_in(
+            available_backups, backup.backup_id, status_filter=status_filter
+        )
+        wal_ranges_to_protect = []
+        while True:
+            if previous_backup is None:
+                # No previous backup so we should remove WALs and return any WAL ranges
+                # we have found so far
+                return True, wal_ranges_to_protect
+            elif (
+                keep_manager.get_keep_target(previous_backup.backup_id)
+                == KeepManager.TARGET_STANDALONE
+            ):
+                # A previous backup exists and it is a standalone backup - if we have
+                # been asked to skip wal cleanup on standalone backups then we
+                # should not remove wals
+                if skip_wal_cleanup_if_standalone:
+                    return False, []
+                # Otherwise we add to the WAL ranges to protect
+                wal_ranges_to_protect.append(
+                    (previous_backup.begin_wal, previous_backup.end_wal)
+                )
+                # and continue iterating through previous backups until we find either
+                # no previous backup or a non-standalone backup
+                previous_backup = BackupManager.find_previous_backup_in(
+                    available_backups,
+                    previous_backup.backup_id,
+                    status_filter=status_filter,
+                )
+                continue
+            else:
+                # A previous backup exists and it is not a standalone backup so we
+                # must not remove any WALs and we can discard any wal_ranges_to_protect
+                # since they are no longer relevant
+                return False, []
+
+    @staticmethod
+    def find_next_backup_in(
+        available_backups, backup_id, status_filter=DEFAULT_STATUS_FILTER
+    ):
+        """
+        Find the next backup (if any) in the supplied dict of BackupInfo objects.
+        """
         ids = sorted(available_backups.keys())
         try:
             current = ids.index(backup_id)
-            while current > 0:
-                res = available_backups[ids[current - 1]]
+            while current < (len(ids) - 1):
+                res = available_backups[ids[current + 1]]
                 if res.status in status_filter:
                     return res
-                current -= 1
+                current += 1
             return None
         except ValueError:
             raise UnknownBackupIdException("Could not find backup_id %s" % backup_id)
@@ -212,17 +345,7 @@ class BackupManager(RemoteStatusMixin):
             status_filter = tuple(status_filter)
         backup = LocalBackupInfo(self.server, backup_id=backup_id)
         available_backups = self.get_available_backups(status_filter + (backup.status,))
-        ids = sorted(available_backups.keys())
-        try:
-            current = ids.index(backup_id)
-            while current < (len(ids) - 1):
-                res = available_backups[ids[current + 1]]
-                if res.status in status_filter:
-                    return res
-                current += 1
-            return None
-        except ValueError:
-            raise UnknownBackupIdException("Could not find backup_id %s" % backup_id)
+        return self.find_next_backup_in(available_backups, backup_id, status_filter)
 
     def get_last_backup_id(self, status_filter=DEFAULT_STATUS_FILTER):
         """
@@ -254,13 +377,52 @@ class BackupManager(RemoteStatusMixin):
         ids = sorted(available_backups.keys())
         return ids[0]
 
-    def delete_backup(self, backup):
+    @staticmethod
+    def get_timelines_to_protect(remove_until, deleted_backup, available_backups):
+        """
+        Returns all timelines in available_backups which are not associated with
+        the backup at remove_until. This is so that we do not delete WALs on
+        any other timelines.
+        """
+        timelines_to_protect = set()
+        # If remove_until is not set there are no backup left
+        if remove_until:
+            # Retrieve the list of extra timelines that contains at least
+            # a backup. On such timelines we don't want to delete any WAL
+            for value in available_backups.values():
+                # Ignore the backup that is being deleted
+                if value == deleted_backup:
+                    continue
+                timelines_to_protect.add(value.timeline)
+            # Remove the timeline of `remove_until` from the list.
+            # We have enough information to safely delete unused WAL files
+            # on it.
+            timelines_to_protect -= set([remove_until.timeline])
+        return timelines_to_protect
+
+    def delete_backup(self, backup, skip_wal_cleanup_if_standalone=True):
         """
         Delete a backup
 
         :param backup: the backup to delete
+        :param bool skip_wal_cleanup_if_standalone: By default we will skip removing
+          WALs if the oldest backups are standalong archival backups (i.e. they have
+          a keep annotation of "standalone"). If this function is being called in the
+          context of a retention policy however, it is safe to set
+          skip_wal_cleanup_if_standalone to False and clean up WALs associated with those
+          backups.
         :return bool: True if deleted, False if could not delete the backup
         """
+        if self.should_keep_backup(backup.backup_id):
+            output.warning(
+                "Skipping delete of backup %s for server %s "
+                "as it has a current keep request. If you really "
+                "want to delete this backup please remove the keep "
+                "and try again.",
+                backup.backup_id,
+                self.config.name,
+            )
+            return False
         available_backups = self.get_available_backups(status_filter=(BackupInfo.DONE,))
         minimum_redundancy = self.server.config.minimum_redundancy
         # Honour minimum required redundancy
@@ -294,7 +456,15 @@ class BackupManager(RemoteStatusMixin):
         output.info(
             "Deleting backup %s for server %s", backup.backup_id, self.config.name
         )
-        previous_backup = self.get_previous_backup(backup.backup_id)
+        should_remove_wals, wal_ranges_to_protect = BackupManager.should_remove_wals(
+            backup,
+            self.get_available_backups(
+                BackupManager.DEFAULT_STATUS_FILTER + (backup.status,)
+            ),
+            keep_manager=self,
+            skip_wal_cleanup_if_standalone=skip_wal_cleanup_if_standalone,
+        )
+
         next_backup = self.get_next_backup(backup.backup_id)
         # Delete all the data contained in the backup
         try:
@@ -307,40 +477,34 @@ class BackupManager(RemoteStatusMixin):
                 e,
             )
             return False
-        # Check if we are deleting the first available backup
-        if not previous_backup:
-            # In the case of exclusive backup (default), removes any WAL
-            # files associated to the backup being deleted.
-            # In the case of concurrent backup, removes only WAL files
-            # prior to the start of the backup being deleted, as they
-            # might be useful to any concurrent backup started immediately
-            # after.
+
+        if should_remove_wals:
+            # There is no previous backup or all previous backups are archival
+            # standalone backups, so we can remove unused WALs (those WALs not
+            # required by standalone archival backups).
+            # If there is a next backup then all unused WALs up to the begin_wal
+            # of the next backup can be removed.
+            # If there is no next backup then there are no remaining backups so:
+            #   - In the case of exclusive backup, remove all unused WAL files.
+            #   - In the case of concurrent backup (the default), removes only
+            #     unused WAL files prior to the start of the backup being deleted,
+            #     as they might be useful to any concurrent backup started
+            #     immediately after.
             remove_until = None  # means to remove all WAL files
             if next_backup:
                 remove_until = next_backup
             elif BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
                 remove_until = backup
 
-            timelines_to_protect = set()
-            # If remove_until is not set there are no backup left
-            if remove_until:
-                # Retrieve the list of extra timelines that contains at least
-                # a backup. On such timelines we don't want to delete any WAL
-                for value in self.get_available_backups(
-                    BackupInfo.STATUS_ARCHIVING
-                ).values():
-                    # Ignore the backup that is being deleted
-                    if value == backup:
-                        continue
-                    timelines_to_protect.add(value.timeline)
-                # Remove the timeline of `remove_until` from the list.
-                # We have enough information to safely delete unused WAL files
-                # on it.
-                timelines_to_protect -= set([remove_until.timeline])
+            timelines_to_protect = self.get_timelines_to_protect(
+                remove_until,
+                backup,
+                self.get_available_backups(BackupInfo.STATUS_ARCHIVING),
+            )
 
             output.info("Delete associated WAL segments:")
             for name in self.remove_wal_before_backup(
-                remove_until, timelines_to_protect
+                remove_until, timelines_to_protect, wal_ranges_to_protect
             ):
                 output.info("\t%s", name)
         # As last action, remove the backup directory,
@@ -565,8 +729,9 @@ class BackupManager(RemoteStatusMixin):
         # Archive every WAL files in the incoming directory of the server
         self.server.archive_wal(verbose=False)
         # Delegate the recovery operation to a RecoveryExecutor object
-        executor = RecoveryExecutor(self)
 
+        command = unix_command_factory(remote_command, self.server.path)
+        executor = recovery_executor_factory(self, command, backup_info.compression)
         # Run the pre_recovery_script if present.
         script = HookScriptRunner(self, "recovery_script", "pre")
         script.env_from_recover(
@@ -647,7 +812,9 @@ class BackupManager(RemoteStatusMixin):
                         "Enforcing retention policy: removing backup %s for "
                         "server %s" % (bid, self.config.name)
                     )
-                    self.delete_backup(available_backups[bid])
+                    self.delete_backup(
+                        available_backups[bid], skip_wal_cleanup_if_standalone=False
+                    )
 
     def delete_basebackup(self, backup):
         """
@@ -919,7 +1086,7 @@ class BackupManager(RemoteStatusMixin):
                             fxlogdb_new.write(wal_info.to_xlogdb_line())
                         else:
                             _logger.warning(
-                                "unexpected file " "rebuilding the wal database: %s",
+                                "unexpected file rebuilding the wal database: %s",
                                 fullname,
                             )
                 fxlogdb_new.flush()
@@ -984,7 +1151,9 @@ class BackupManager(RemoteStatusMixin):
         # Return the timeline map
         return timelines
 
-    def remove_wal_before_backup(self, backup_info, timelines_to_protect=None):
+    def remove_wal_before_backup(
+        self, backup_info, timelines_to_protect=None, wal_ranges_to_protect=[]
+    ):
         """
         Remove WAL files which have been archived before the start of
         the provided backup.
@@ -997,6 +1166,8 @@ class BackupManager(RemoteStatusMixin):
         :param BackupInfo|None backup_info: the backup information structure
         :param set timelines_to_protect: optional list of timelines
             to protect
+        :param list wal_ranges_to_protect: optional list of `(begin_wal, end_wal)`
+            tuples which define inclusive ranges of WALs which must not be deleted.
         :return list: a list of removed WAL files
         """
         removed = []
@@ -1024,6 +1195,16 @@ class BackupManager(RemoteStatusMixin):
                         tli, _, _ = xlog.decode_segment_name(wal_info.name)
                         keep |= tli in timelines_to_protect
 
+                    # Keeps the WAL segment if it is within a protected range
+                    if xlog.is_backup_file(wal_info.name):
+                        # If we have a .backup file then truncate the name for the
+                        # range check
+                        wal_name = wal_info.name[:24]
+                    else:
+                        wal_name = wal_info.name
+                    for begin_wal, end_wal in wal_ranges_to_protect:
+                        keep |= wal_name >= begin_wal and wal_name <= end_wal
+
                     # Keeps the WAL segment if it is a newer
                     # than the given backup (the first available)
                     if backup_info and backup_info.begin_wal is not None:
@@ -1076,6 +1257,32 @@ class BackupManager(RemoteStatusMixin):
             # If no backup is available return false
             return False, "No available backups"
 
+    def validate_last_backup_min_size(self, last_backup_minimum_size):
+        """
+        Evaluate the size of the last available backup in a catalogue.
+        If the last backup is smaller than the specified size
+        the function returns False.
+        Otherwise, the function returns True.
+
+        :param last_backup_minimum_size: size in bytes
+            representing the maximum allowed age for the last backup
+            in a server catalogue
+        :return tuple: a tuple containing the boolean result of the check and
+            auxiliary information about the last backup current age
+        """
+        # Get the ID of the last available backup
+        backup_id = self.get_last_backup_id()
+        if backup_id:
+            # Get the backup object
+            backup = LocalBackupInfo(self.server, backup_id=backup_id)
+            if backup.size < last_backup_minimum_size:
+                return False, backup.size
+            else:
+                return True, backup.size
+        else:
+            # If no backup is available return false
+            return False, 0
+
     def backup_fsync_and_set_sizes(self, backup_info):
         """
         Fsync all files in a backup and set the actual size on disk
@@ -1211,3 +1418,34 @@ class BackupManager(RemoteStatusMixin):
             # all the WAL files until that point are present.
             backup_info.status = BackupInfo.WAITING_FOR_WALS
         backup_info.save()
+
+    def verify_backup(self, backup_info):
+        """
+        This function should check if pg_verifybackup is installed and run it against backup path
+        should test if pg_verifybackup is installed locally
+
+
+        :param backup_info: barman.infofile.LocalBackupInfo instance
+        """
+        output.info("Calling pg_verifybackup")
+        # Test pg_verifybackup existence
+        version_info = PgVerifyBackup.get_version_info(self.server.path)
+        if version_info.get("full_path", None) is None:
+            output.error("pg_verifybackup not found")
+            return
+
+        pg_verifybackup = PgVerifyBackup(
+            data_path=backup_info.get_data_directory(),
+            command=version_info["full_path"],
+            version=version_info["full_version"],
+        )
+        try:
+            pg_verifybackup()
+        except CommandFailedException as e:
+            output.error(
+                "verify backup failure on directory '%s'"
+                % backup_info.get_data_directory()
+            )
+            output.error(e.args[0]["err"])
+            return
+        output.info(pg_verifybackup.get_output()[0].strip())
