@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -32,6 +32,7 @@ import logging
 import os
 import re
 import shutil
+from tarfile import TarFile
 from abc import ABCMeta, abstractmethod
 from functools import partial
 
@@ -40,9 +41,11 @@ from distutils.version import LooseVersion as Version
 
 from barman import output, xlog
 from barman.command_wrappers import PgBaseBackup
+from barman.compression import get_pg_basebackup_compression
 from barman.config import BackupOptions
 from barman.copy_controller import RsyncCopyController
 from barman.exceptions import (
+    BackupException,
     CommandFailedException,
     DataTransferFailure,
     FsOperationFailed,
@@ -165,12 +168,12 @@ class BackupExecutor(with_metaclass(ABCMeta, RemoteStatusMixin)):
             if removed:
                 # report the list of the removed WAL files
                 output.info(
-                    "WAL segments preceding the current backup " "have been found:",
+                    "WAL segments preceding the current backup have been found:",
                     log=False,
                 )
                 for wal_name in removed:
                     output.info(
-                        "\t%s from server %s " "has been removed",
+                        "\t%s from server %s has been removed",
                         wal_name,
                         self.config.name,
                     )
@@ -232,8 +235,11 @@ class PostgresBackupExecutor(BackupExecutor):
             assigned to the executor
         """
         super(PostgresBackupExecutor, self).__init__(backup_manager, "postgres")
+        self.backup_compression = get_pg_basebackup_compression(self.server)
         self.validate_configuration()
-        self.strategy = PostgresBackupStrategy(self.server.postgres, self.config.name)
+        self.strategy = PostgresBackupStrategy(
+            self.server.postgres, self.config.name, self.backup_compression
+        )
 
     def validate_configuration(self):
         """
@@ -276,7 +282,7 @@ class PostgresBackupExecutor(BackupExecutor):
             self.server.config.disabled = True
             # Report the error in the configuration errors message list
             self.server.config.msg_list.append(
-                "reuse_backup option is not supported by " "postgres backup_method"
+                "reuse_backup option is not supported by postgres backup_method"
             )
 
         # Forbid network_compression option.
@@ -289,6 +295,7 @@ class PostgresBackupExecutor(BackupExecutor):
                 "postgres backup_method"
             )
 
+        remote_status = None
         # bandwidth_limit option is supported by pg_basebackup executable
         # starting from Postgres 9.4
         if self.server.config.bandwidth_limit:
@@ -307,6 +314,12 @@ class PostgresBackupExecutor(BackupExecutor):
                     % remote_status["pg_basebackup_version"]
                 )
 
+        # validate compression options
+        if self.backup_compression:
+            if remote_status is None:
+                remote_status = self.fetch_remote_status()
+            self.backup_compression.validate(self.server, remote_status)
+
     def backup(self, backup_info):
         """
         Perform a backup for the server - invoked by BackupManager.backup()
@@ -418,9 +431,7 @@ class PostgresBackupExecutor(BackupExecutor):
                     "and PostgreSQL older than 9.3"
                 )
             else:
-                hint = (
-                    "pg_basebackup 9.4 or higher is required for " "tablespaces support"
-                )
+                hint = "pg_basebackup 9.4 or higher is required for tablespaces support"
 
         # Even if pg_basebackup doesn't support the tablespace-mapping
         # option, this location can be correctly backed up as doesn't
@@ -433,9 +444,7 @@ class PostgresBackupExecutor(BackupExecutor):
                     "support is not required"
                 )
             else:
-                hint = (
-                    "pg_basebackup 9.4 or higher is required for " "tablespaces support"
-                )
+                hint = "pg_basebackup 9.4 or higher is required for tablespaces support"
 
         check_strategy.result(self.config.name, check_result, hint=hint)
 
@@ -574,6 +583,7 @@ class PostgresBackupExecutor(BackupExecutor):
             retry_times=self.config.basebackup_retry_times,
             retry_sleep=self.config.basebackup_retry_sleep,
             retry_handler=partial(self._retry_handler, dest_dirs),
+            compression=self.backup_compression,
         )
 
         # Do the actual copy
@@ -628,7 +638,7 @@ class PostgresBackupExecutor(BackupExecutor):
             failure
         """
         output.warning(
-            "Failure executing a backup using pg_basebackup " "(attempt %s)", attempt
+            "Failure executing a backup using pg_basebackup (attempt %s)", attempt
         )
         output.warning(
             "The files copied so far will be removed and "
@@ -716,17 +726,12 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
         concurrent_backup = BackupOptions.CONCURRENT_BACKUP in backup_options
         exclusive_backup = BackupOptions.EXCLUSIVE_BACKUP in backup_options
         if not concurrent_backup and not exclusive_backup:
-            self.config.backup_options.add(BackupOptions.EXCLUSIVE_BACKUP)
+            self.config.backup_options.add(BackupOptions.CONCURRENT_BACKUP)
             output.warning(
                 "No backup strategy set for server '%s' "
-                "(using default 'exclusive_backup').",
+                "(using default 'concurrent_backup').",
                 self.config.name,
             )
-            output.warning(
-                "The default backup strategy will change "
-                "to 'concurrent_backup' in the future. "
-                "Explicitly set 'backup_options' to silence this warning."
-            )
 
         # Depending on the backup options value, create the proper strategy
         if BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
@@ -1108,6 +1113,15 @@ class RsyncBackupExecutor(FsBackupExecutor):
             assigned to the strategy
         """
         super(RsyncBackupExecutor, self).__init__(backup_manager, "rsync", local_mode)
+        self.validate_configuration()
+
+    def validate_configuration(self):
+        # Verify that backup_compression is not set
+        if self.server.config.backup_compression:
+            self.server.config.disabled = True
+            self.server.config.msg_list.append(
+                "backup_compression option is not supported by rsync backup_method"
+            )
 
     def backup_copy(self, backup_info):
         """
@@ -1421,7 +1435,7 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
         :param barman.infofile.BackupInfo backup_info: object
         representing a
             backup
-        :param DictCursor start_info: the result of the pg_start_backup
+        :param DictCursor start_info: the result of the pg_backup_start
         command
         """
         backup_info.set_attribute("status", BackupInfo.STARTED)
@@ -1513,9 +1527,13 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
             "begin_offset",
             xlog.parse_lsn(wal_info.group(1)) % backup_info.xlog_segment_size,
         )
-        backup_info.set_attribute(
-            "begin_time", dateutil.parser.parse(start_time.group(1))
-        )
+
+        # If we have already obtained a begin_time then it takes precedence over the
+        # begin time in the backup label
+        if not backup_info.begin_time:
+            backup_info.set_attribute(
+                "begin_time", dateutil.parser.parse(start_time.group(1))
+            )
 
     def _read_backup_label(self, backup_info):
         """
@@ -1527,7 +1545,8 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
         label_path = os.path.join(backup_info.get_data_directory(), "backup_label")
         output.debug("Reading backup label: %s" % label_path)
         with open(label_path, "r") as f:
-            backup_info.set_attribute("backup_label", f.read())
+            backup_label = f.read()
+        backup_info.set_attribute("backup_label", backup_label)
 
 
 class PostgresBackupStrategy(BackupStrategy):
@@ -1539,6 +1558,19 @@ class PostgresBackupStrategy(BackupStrategy):
     using pg_basebackup.
     """
 
+    def __init__(self, postgres, server_name, backup_compression=None):
+        """
+        Constructor
+
+        :param barman.postgres.PostgreSQLConnection postgres: the PostgreSQL
+            connection
+        :param str server_name: The name of the server
+        :param barman.compression.PgBaseBackupCompression backup_compression:
+            the pg_basebackup compression options used for this backup
+        """
+        super(PostgresBackupStrategy, self).__init__(postgres, server_name)
+        self.backup_compression = backup_compression
+
     def check(self, check_strategy):
         """
         Perform additional checks for the Postgres backup strategy
@@ -1572,6 +1604,9 @@ class PostgresBackupStrategy(BackupStrategy):
 
         :param barman.infofile.LocalBackupInfo backup_info: backup information
         """
+        if self.backup_compression and self.backup_compression.format != "plain":
+            backup_info.set_attribute("compression", self.backup_compression.type)
+
         self._read_backup_label(backup_info)
         self._backup_info_from_backup_label(backup_info)
 
@@ -1593,6 +1628,40 @@ class PostgresBackupStrategy(BackupStrategy):
             # Skip switching XLOG if a standby server
             pass
 
+    def _read_compressed_backup_label(self, backup_info):
+        """
+        Read the contents of a backup_label file from a compressed archive.
+
+        :param barman.infofile.LocalBackupInfo backup_info: backup information
+        """
+        basename = os.path.join(backup_info.get_data_directory(), "base.tar")
+        with self.backup_compression.open(basename) as f:
+            self.tar = TarFile.open(fileobj=f, mode="r|")
+            for member in self.tar:
+                if member.name == "backup_label":
+                    backup_label_fileobj = self.tar.extractfile(member)
+                    return backup_label_fileobj.read().decode("utf-8")
+            raise BackupException(
+                "Could not find backup_label in %s"
+                % self.backup_compression.with_suffix(basename)
+            )
+
+    def _read_backup_label(self, backup_info):
+        """
+        Read the backup_label file.
+
+        Transparently handles the fact that the backup_label file may be in a
+        compressed tarball.
+
+        :param barman.infofile.LocalBackupInfo backup_info: backup information
+        """
+        self.current_action = "reading the backup label"
+        if backup_info.compression is not None:
+            backup_label = self._read_compressed_backup_label(backup_info)
+            backup_info.set_attribute("backup_label", backup_label)
+        else:
+            super(PostgresBackupStrategy, self)._read_backup_label(backup_info)
+
 
 class ExclusiveBackupStrategy(BackupStrategy):
     """
@@ -1669,6 +1738,23 @@ class ExclusiveBackupStrategy(BackupStrategy):
                     False,
                     hint="cannot perform exclusive backup on a standby",
                 )
+        check_strategy.init_check("exclusive backup supported")
+        try:
+            if self.postgres and self.postgres.server_version < 150000:
+                check_strategy.result(self.server_name, True)
+            else:
+                check_strategy.result(
+                    self.server_name,
+                    False,
+                    hint="exclusive backups not supported "
+                    "on PostgreSQL %s" % self.postgres.server_major_version,
+                )
+        except PostgresConnectionError:
+            check_strategy.result(
+                self.server_name,
+                False,
+                hint="unable to determine postgres version",
+            )
 
 
 class ConcurrentBackupStrategy(BackupStrategy):
@@ -1749,8 +1835,8 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
         :param barman.infofile.BackupInfo backup_info: backup information
         """
-        pg_version = self.postgres.server_version
         self.current_action = "issuing stop backup command"
+        pg_version = self.postgres.server_version
         if pg_version >= 90600:
             # On 9.6+ execute native concurrent stop backup
             self.current_action += " (native concurrent)"
