@@ -37,7 +37,7 @@ import barman.config
 import barman.diagnose
 from barman import output
 from barman.annotations import KeepManager
-from barman.config import RecoveryOptions
+from barman.config import RecoveryOptions, parse_recovery_staging_path
 from barman.exceptions import (
     BadXlogSegmentName,
     RecoveryException,
@@ -50,6 +50,7 @@ from barman.utils import (
     BarmanEncoder,
     check_non_negative,
     check_positive,
+    check_tli,
     configure_logging,
     drop_privileges,
     force_str,
@@ -441,7 +442,11 @@ def backup(args):
         if args.retry_times is not None:
             server.config.basebackup_retry_times = args.retry_times
         if hasattr(args, "immediate_checkpoint"):
+            # As well as overriding the immediate_checkpoint value in the config
+            # we must also update the immediate_checkpoint attribute on the
+            # postgres connection because it has already been set from the config
             server.config.immediate_checkpoint = args.immediate_checkpoint
+            server.postgres.immediate_checkpoint = args.immediate_checkpoint
         if args.jobs is not None:
             server.config.parallel_jobs = args.jobs
         if hasattr(args, "bwlimit"):
@@ -583,7 +588,7 @@ def rebuild_xlogdb(args):
             completer=server_completer,
             help="specifies the server name for the command ",
         ),
-        argument("--target-tli", help="target timeline", type=check_positive),
+        argument("--target-tli", help="target timeline", type=check_tli),
         argument(
             "--target-time",
             help="target time. You can use any valid unambiguous representation. "
@@ -708,6 +713,17 @@ def rebuild_xlogdb(args):
             default=SUPPRESS,
             help="Enable standby mode when starting the recovered PostgreSQL instance",
         ),
+        argument(
+            "--recovery-staging-path",
+            dest="recovery_staging_path",
+            help=(
+                "A path to a location on the recovery host where compressed backup "
+                "files will be staged during the recovery. This location must have "
+                "enough available space to temporarily hold the full compressed "
+                "backup. This option is *required* when recovering from a compressed "
+                "backup."
+            ),
+        ),
     ]
 )
 def recover(args):
@@ -727,6 +743,35 @@ def recover(args):
         )
         output.close_and_exit()
 
+    # If the backup to be recovered is compressed then there are additional
+    # checks to be carried out
+    if backup_id.compression is not None:
+        # Set the recovery staging path from the cli if it is set
+        if args.recovery_staging_path is not None:
+            try:
+                recovery_staging_path = parse_recovery_staging_path(
+                    args.recovery_staging_path
+                )
+            except ValueError as exc:
+                output.error("Cannot parse recovery staging path: %s", str(exc))
+                output.close_and_exit()
+            server.config.recovery_staging_path = recovery_staging_path
+        # If the backup is compressed but there is no recovery_staging_path
+        # then this is an error - the user *must* tell barman where recovery
+        # data can be staged.
+        if server.config.recovery_staging_path is None:
+            output.error(
+                "Cannot recover from backup '%s' of server '%s': "
+                "backup is compressed with %s compression but no recovery "
+                "staging path is provided. Either set recovery_staging_path "
+                "in the Barman config or use the --recovery-staging-path "
+                "argument.",
+                args.backup_id,
+                server.config.name,
+                backup_id.compression,
+            )
+            output.close_and_exit()
+
     # decode the tablespace relocation rules
     tablespaces = {}
     if args.tablespace:
@@ -775,7 +820,7 @@ def recover(args):
     if hasattr(args, "get_wal"):
         if args.get_wal:
             server.config.recovery_options.add(RecoveryOptions.GET_WAL)
-        else:
+        elif RecoveryOptions.GET_WAL in server.config.recovery_options:
             server.config.recovery_options.remove(RecoveryOptions.GET_WAL)
     if args.jobs is not None:
         server.config.parallel_jobs = args.jobs
@@ -872,11 +917,13 @@ def show_servers(args):
 
         # If the server has been manually disabled
         if not server.config.active:
-            name += " (inactive)"
+            description = "(inactive)"
         # If server has configuration errors
         elif server.config.disabled:
-            name += " (WARNING: disabled)"
-        output.init("show_server", name)
+            description = "(WARNING: disabled)"
+        else:
+            description = None
+        output.init("show_server", name, description=description)
         with closing(server):
             server.show()
     output.close_and_exit()
