@@ -70,6 +70,9 @@ BACKUP_METHOD_VALUES = ["rsync", "postgres", "local-rsync"]
 
 CREATE_SLOT_VALUES = ["manual", "auto"]
 
+# Config values relating to pg_basebackup compression
+BASEBACKUP_COMPRESSIONS = ["gzip"]
+
 
 class CsvOption(set):
 
@@ -287,6 +290,56 @@ def parse_reuse_backup(value):
     )
 
 
+def parse_backup_compression(value):
+    """
+    Parse a string to a valid backup_compression value.
+
+    Valid values are contained in compression.basebackup_compressions list
+
+    :param str value: backup_compression value
+    :raises ValueError: if the value is invalid
+    """
+    if value is None:
+        return None
+    if value.lower() in BASEBACKUP_COMPRESSIONS:
+        return value.lower()
+    raise ValueError(
+        "Invalid value (must be one in: '%s')" % ("', '".join(BASEBACKUP_COMPRESSIONS))
+    )
+
+
+def parse_backup_compression_format(value):
+    """
+    Parse a string to a valid backup_compression format value.
+
+    Valid values are "plain" and "tar"
+
+    :param str value: backup_compression_location value
+    :raises ValueError: if the value is invalid
+    """
+    if value is None:
+        return None
+    if value.lower() in ("plain", "tar"):
+        return value.lower()
+    raise ValueError("Invalid value (must be either `plain` or `tar`)")
+
+
+def parse_backup_compression_location(value):
+    """
+    Parse a string to a valid backup_compression location value.
+
+    Valid values are "client" and "server"
+
+    :param str value: backup_compression_location value
+    :raises ValueError: if the value is invalid
+    """
+    if value is None:
+        return None
+    if value.lower() in ("client", "server"):
+        return value.lower()
+    raise ValueError("Invalid value (must be either `client` or `server`)")
+
+
 def parse_backup_method(value):
     """
     Parse a string to a valid backup_method value.
@@ -305,6 +358,12 @@ def parse_backup_method(value):
     )
 
 
+def parse_recovery_staging_path(value):
+    if value is None or os.path.isabs(value):
+        return value
+    raise ValueError("Invalid value : '%s' (must be an absolute path)" % value)
+
+
 def parse_slot_name(value):
     """
     Replication slot names may only contain lower case letters, numbers,
@@ -354,6 +413,10 @@ class ServerConfig(object):
         "active",
         "archiver",
         "archiver_batch_size",
+        "backup_compression",
+        "backup_compression_format",
+        "backup_compression_level",
+        "backup_compression_location",
         "backup_directory",
         "backup_method",
         "backup_options",
@@ -403,6 +466,7 @@ class ServerConfig(object):
         "pre_wal_delete_retry_script",
         "primary_ssh_command",
         "recovery_options",
+        "recovery_staging_path",
         "create_slot",
         "retention_policy",
         "retention_policy_mode",
@@ -423,6 +487,10 @@ class ServerConfig(object):
     BARMAN_KEYS = [
         "archiver",
         "archiver_batch_size",
+        "backup_compression",
+        "backup_compression_format",
+        "backup_compression_level",
+        "backup_compression_location",
         "backup_method",
         "backup_options",
         "bandwidth_limit",
@@ -466,6 +534,7 @@ class ServerConfig(object):
         "pre_wal_delete_retry_script",
         "primary_ssh_command",
         "recovery_options",
+        "recovery_staging_path",
         "create_slot",
         "retention_policy",
         "retention_policy_mode",
@@ -519,6 +588,10 @@ class ServerConfig(object):
         "active": parse_boolean,
         "archiver": parse_boolean,
         "archiver_batch_size": int,
+        "backup_compression": parse_backup_compression,
+        "backup_compression_format": parse_backup_compression_format,
+        "backup_compression_level": int,
+        "backup_compression_location": parse_backup_compression_location,
         "backup_method": parse_backup_method,
         "backup_options": BackupOptions,
         "basebackup_retry_sleep": int,
@@ -534,6 +607,7 @@ class ServerConfig(object):
         "network_compression": parse_boolean,
         "parallel_jobs": int,
         "recovery_options": RecoveryOptions,
+        "recovery_staging_path": parse_recovery_staging_path,
         "create_slot": parse_create_slot,
         "reuse_backup": parse_reuse_backup,
         "streaming_archiver": parse_boolean,
@@ -940,6 +1014,11 @@ class Config(object):
         """
         # Check for the existence of unexpected parameters in the
         # global section of the configuration file
+        required_keys = [
+            "barman_home",
+        ]
+        self._detect_missing_keys(self._global_config, required_keys, "barman")
+
         keys = [
             "barman_home",
             "barman_lock_directory",
@@ -961,6 +1040,31 @@ class Config(object):
         # server section of the configuration file
         self._validate_with_keys(self._config.items(server), ServerConfig.KEYS, server)
 
+    @staticmethod
+    def _detect_missing_keys(config_items, required_keys, section):
+        """
+        Check config for any missing required keys
+
+        :param config_items: list of tuples containing provided parameters
+            along with their values
+        :param required_keys: list of required keys
+        :param section: source section (for error reporting)
+        """
+        missing_key_detected = False
+
+        config_keys = [item[0] for item in config_items]
+        for req_key in required_keys:
+            # if a required key is not found, then print an error
+            if req_key not in config_keys:
+                output.error(
+                    'Parameter "%s" is required in [%s] section.' % (req_key, section),
+                )
+                missing_key_detected = True
+        if missing_key_detected:
+            raise SystemExit(
+                "Your configuration is missing required parameters. Exiting."
+            )
+
     @staticmethod
     def _validate_with_keys(config_items, allowed_keys, section):
         """
