@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -263,6 +263,13 @@ class PostgresUnsupportedFeature(PostgresException):
     """
 
 
+class PostgresObsoleteFeature(PostgresException):
+    """
+    Obsolete feature, i.e. one which has been deprecated and since
+    removed.
+    """
+
+
 class PostgresDuplicateReplicationSlot(PostgresException):
     """
     The creation of a physical replication slot failed because
@@ -367,3 +374,21 @@ class UnrecoverableHookScriptError(BarmanException):
     """
     Exception for hook script errors which mean the script should not be retried.
     """
+
+
+class ArchivalBackupException(BarmanException):
+    """
+    Exception for errors concerning archival backups.
+    """
+
+
+class WalArchiveContentError(BarmanException):
+    """
+    Exception raised when unexpected content is detected in the WAL archive.
+    """
+
+
+class InvalidRetentionPolicy(BarmanException):
+    """
+    Exception raised when a retention policy cannot be parsed.
+    """
