@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2013-2021
+# © Copyright EnterpriseDB UK Limited 2013-2022
 #
 # This file is part of Barman.
 #
@@ -25,7 +25,6 @@ import os
 import dateutil.parser
 import dateutil.tz
 
-import barman.compression
 from barman import xlog
 from barman.exceptions import BackupInfoBadInitialisation
 from barman.utils import fsync_dir
@@ -323,12 +322,14 @@ class WalFileInfo(FieldListFile):
     name = Field("name", doc="base name of WAL file")
     size = Field("size", load=int, doc="WAL file size after compression")
     time = Field(
-        "time", load=float, doc="WAL file modification time " "(seconds since epoch)"
+        "time", load=float, doc="WAL file modification time (seconds since epoch)"
     )
     compression = Field("compression", doc="compression type")
 
     @classmethod
-    def from_file(cls, filename, unidentified_compression=None, **kwargs):
+    def from_file(
+        cls, filename, compression_manager=None, unidentified_compression=None, **kwargs
+    ):
         """
         Factory method to generate a WalFileInfo from a WAL file.
 
@@ -337,17 +338,19 @@ class WalFileInfo(FieldListFile):
         an AttributeError exception is raised.
 
         :param str filename: the file to inspect
+        :param Compressionmanager compression_manager: a compression manager
+            which will be used to identify the compression
         :param str unidentified_compression: the compression to set if
-            the current schema is not identifiable.
+            the current schema is not identifiable
         """
-        identify_compression = barman.compression.identify_compression
         stat = os.stat(filename)
         kwargs.setdefault("name", os.path.basename(filename))
         kwargs.setdefault("size", stat.st_size)
         kwargs.setdefault("time", stat.st_mtime)
         if "compression" not in kwargs:
             kwargs["compression"] = (
-                identify_compression(filename) or unidentified_compression
+                compression_manager.identify_compression(filename)
+                or unidentified_compression
             )
         obj = cls(**kwargs)
         obj.filename = "%s.meta" % filename
@@ -424,7 +427,16 @@ class BackupInfo(FieldListFile):
     VALID = "VALID"
     POTENTIALLY_OBSOLETE = "OBSOLETE*"
     NONE = "-"
-    RETENTION_STATUS = (OBSOLETE, VALID, POTENTIALLY_OBSOLETE, NONE)
+    KEEP_FULL = "KEEP:FULL"
+    KEEP_STANDALONE = "KEEP:STANDALONE"
+    RETENTION_STATUS = (
+        OBSOLETE,
+        VALID,
+        POTENTIALLY_OBSOLETE,
+        KEEP_FULL,
+        KEEP_STANDALONE,
+        NONE,
+    )
 
     version = Field("version", load=int)
     pgdata = Field("pgdata")
@@ -459,6 +471,7 @@ class BackupInfo(FieldListFile):
         "xlog_segment_size", load=int, default=xlog.DEFAULT_XLOG_SEG_SIZE
     )
     systemid = Field("systemid")
+    compression = Field("compression")
 
     __slots__ = "backup_id", "backup_version"
 
@@ -647,7 +660,7 @@ class LocalBackupInfo(BackupInfo):
                 self.backup_version = 1
         except Exception as e:
             _logger.warning(
-                "Error detecting backup_version, " "use default: 2. Failure reason: %s",
+                "Error detecting backup_version, use default: 2. Failure reason: %s",
                 e,
             )
 
@@ -658,6 +671,7 @@ class LocalBackupInfo(BackupInfo):
         # Walk down the base backup directory
         if target in ("data", "standalone", "full"):
             for root, _, files in os.walk(self.get_basebackup_directory()):
+                files.sort()
                 for f in files:
                     yield os.path.join(root, f)
         if target in "standalone":
